#ifndef ALPHABET_H
#define ALPHABET_H

/**
 * This class implements the Alphabet: a simple container for
 * (typed) constants.
 *
 * The Alphabet is where all the constants are defined.  Any Constant
 * occuring in any term is a pointer to an element of the alphabet.
 * Hence, constants are never deleted, unless the alphabet is cleaned
 * up.
 *
 * Copying the Alphabet does not do a deep copy.  As such, you should
 * only call free_memory() on an Alphabet if you are really done with
 * it, and don't have any terms / copies still referring to constants
 * in the Alphabet.
 */

#include <map>
#include "term.h"

class Alphabet : public map<string,Constant*> {
  public:
    bool contains(string key);
    void add(Constant* c);
    void free_memory();
    string to_string();
};

#endif

