#include "environment.h"

string Environment :: choose_new_name(Alphabet sigma, PType type, bool bound) {
  char good_bound[] = { 'x','y','z','u','v','w','p','q','r','s','t','a','b' };
  char good_free_base[] = { 'X','Y','Z','U','V','W','P','Q','R','S','T' };
  char good_free_composed[] = { 'F','G','H', 'I', 'J' };

  char* good = bound ? good_bound : (type->query_base() ? good_free_base : good_free_composed);
  int Ngood = bound ? 13 : (type->query_base() ? 11 : 5);

  string chosen = "";
  for (int i = 0; i < Ngood; i++) {
    chosen = string(1, good[i]);
    if (!contains(chosen) && !sigma.contains(chosen)) break;
    else chosen = "";
  }
  if (chosen == "") {
    chosen = string(1, good[random() % Ngood]);
  }

  // we have a start letter - append more as necessary
  while (contains(chosen) || sigma.contains(chosen)) {
    char followup = 'a' + random() % 26;
    if (!bound) followup += 'A'-'a';

    for (char j = 'a'; j <= 'z'; j++) {
      char jj = bound ? j : j + ('A'-'a');
      if (!contains(chosen + jj) && !sigma.contains(chosen)) {followup = jj; break;}
    }
    
    chosen = chosen + followup;
  }
  
  return chosen;
}

void Environment :: add(Variable* v) {
  insert(pair<string,Variable*>(v->to_string(), v));
}

string Environment :: add(PType type, Alphabet sigma, bool bound) {
  string name = choose_new_name(sigma, type, bound);
  add(new Variable(name, type));
  return name;
}

void Environment :: remove(string key) {
  erase(key);
}

void Environment :: combine(Environment other) {
  map<string,Variable*>::iterator it;

  for (it = other.begin(); it != other.end(); it++) {
    if (!contains(it->first)) add(it->second);
  }
}

void Environment :: setminus(Environment other) {
  map<string,Variable*>::iterator it;

  for (it = other.begin(); it != other.end(); it++) {
    if (contains(it->first)) remove(it->first);
  }
}

bool Environment :: contains(string key) {
  return find(key) != end();
}

bool Environment :: subset(Environment other) {
  map<string,Variable*>::iterator it;
  for (it = begin(); it != end(); it++) {
    if (!other.contains(it->first)) return false;
    if (other[it->first] != it->second) return false;
  }
  return true;
}

void Environment :: free_memory() {
  map<string,Variable*>::iterator it;

  for (it = begin(); it != end(); it++) {
    delete it->second;
  }
  clear();
}

string Environment :: to_string() {
  map<string,Variable*>::iterator it;
  string ret;
  
  ret = "{";
  for (it = begin(); it != end(); it++) {
    ret = ret + it->second->to_string_complete() + ", ";
  }
  return ret.substr(0,ret.length()-2) + "}";
}

