/**
 * The MatchRule class forms the basis for standard user defined
 * rules.  A term s can be rewritten to t by matchrule l->r, if
 * there is a substitution sigma such that s = l sigma, t =
 * r sigma.
 */

#include "term.h"
#include "environment.h"
#include "rule.h"

class MatchRule : public Rule {
  private:
    bool match_left(PTerm left, PTerm with, Substitution &sub,
                    Environment bound);

  protected:
    Environment gamma;
    PTerm l, r;
    bool valid;
    string invalid_reason;

    void set_valid(bool value);

    virtual void check_valid();
    virtual bool matches(PTerm s, Substitution &match);
    virtual PTerm apply_top(PTerm term);
    virtual bool applicable_top(PTerm term);

  public:
    MatchRule();
    MatchRule(Environment _gamma, PTerm _l, PTerm _r);

    bool query_valid();
    string query_invalid_reason();
    void free_memory();
      // frees l and and as well
    string to_string();
};

