#include "rule.h"
#include "term.h"
#include "substitution.h"

/* === Rule inherit === */

PTerm Rule :: apply_top(PTerm term) {
  return term;
}

bool Rule :: applicable_top(PTerm term) {
  return false;
}

PTerm Rule :: apply(PTerm term, string position) {
  if (term == NULL) return NULL;

  if (position == "") return apply_top(term);

  PTerm sub = term->subterm(position);
  if (sub == NULL) return term;
  sub = apply_top(sub);
  term->replace_subterm(sub, position);
  return term;
}

bool Rule :: applicable(PTerm term, string position) {
  if (term == NULL) return false;

  PTerm sub = term->subterm(position);
  if (sub == NULL) return false;
  return applicable_top(sub);
}

PTerm Rule :: normalise(PTerm term) {
  // normalise children
  if (term->query_abstraction()) {
    PTerm sub = normalise(term->subterm("1"));
    term->replace_subterm(sub, "1");
  }
  if (term->query_application()) {
    PTerm left = normalise(term->subterm("1"));
    PTerm right = normalise(term->subterm("2"));
    term->replace_subterm(left, "1");
    term->replace_subterm(right, "2");
  }

  // apply top rule and continue normalising if possible
  if (applicable_top(term)) {
    return normalise(apply_top(term));
  }
  else {
    return term;
  }
}

bool Rule :: normal(PTerm term) {
  PTerm sub1, sub2;

  if (term == NULL) return true;
  if (applicable_top(term)) return false;
  
  sub1 = term->subterm("1");
  sub2 = term->subterm("2");
  return normal(sub1) && normal(sub2);
}

string Rule :: to_string() {
  return name;
}

