#ifndef TERMREWRITESYSTEM_H
#define TERMREWRITESYSTEM_H

/**
 * TermRewriteSystem: a wrapper class for a term rewrite system.
 *
 * This can be used (or inherited) for different kinds of system, as no
 * assumptions are made on the kinds of rules in the system.
 */

#include <list>
#include "alphabet.h"
#include "rule.h"

typedef list<Rule*> Ruleset;

class TermRewriteSystem {
  private:
    Alphabet alphabet;
    Ruleset rules;

  public:
    TermRewriteSystem() {}
    TermRewriteSystem(Alphabet _alphabet) : alphabet(_alphabet) {}
    TermRewriteSystem(Alphabet _alphabet, Ruleset _rules)
      : alphabet(_alphabet), rules(_rules) {}

    Alphabet query_alphabet() {return alphabet;}
    Ruleset query_rules() {return rules;}

    void set_alphabet(Alphabet _alphabet) {alphabet = _alphabet;}
    void set_rules(Ruleset _rules) {rules = _rules;}

    void add_rule(Rule* rule) {rules.push_back(rule);}
};

#endif

